﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class CameraScaler : MonoBehaviour {

    /// <summary>
    /// The size of the playing field
    /// </summary>
    public static int cameraScale = 10;     // Default size is 10

    /// <summary>
    /// The text that displays the size of the playing field.
    /// </summary>
    public Text displayText;

    Camera mainCamera;

    bool isShowing = false;

	// Use this for initialization
	void Start ()
    {
        mainCamera = Camera.main;
        if (mainCamera.orthographic)
        {
            cameraScale = PlayerManager.PlayerCount * 4;
        }
        else
        {
            cameraScale = PlayerManager.PlayerCount * -5 - 10;
        }
	}
	
	// Update is called once per frame
	void Update ()
    {
        // Size the playing field by sizing the camera
	    if (Input.GetKeyDown(KeyCode.Equals) || Input.GetKeyDown(KeyCode.Plus))   // The one that also has the plus sign :P
        {
            cameraScale++;
        }
        else if (Input.GetKeyDown(KeyCode.Minus))
        {
            cameraScale--;
        }
        if (mainCamera.orthographic)
        {
            mainCamera.orthographicSize = cameraScale;
        }
        else
        {
            mainCamera.transform.position = new Vector3(mainCamera.transform.position.x,
                        mainCamera.transform.position.y, cameraScale);
        }
        StartCoroutine(DisplaySize(1.0f));
	}

    /// <summary>
    /// Display the camera scale for x seconds
    /// </summary>
    /// <param name="seconds">The amount of seconds the size is displayed</param>
    /// <returns></returns>
    IEnumerator DisplaySize(float seconds)
    {
        while(isShowing)
        {
            yield return null;
        }
        isShowing = true;
        displayText.text = "" + cameraScale;
        yield return new WaitForSeconds(seconds);
        displayText.text = "";
        isShowing = false;
    }
}
