﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System;

public class CountdownManager : MonoBehaviour {

    /// <summary>
    /// Strings shown before the countdown is over
    /// </summary>
    public string[] countdownText = new string[]
    {
        "3",
        "2",
        "1",
        "Start!"
    };

    /// <summary>
    /// Audioclips for the 3 2 1 Go! Sound.
    /// </summary>
    public AudioClip numbersSound;
    public AudioClip goSound;

    /// <summary>
    /// The text component that displays everyting
    /// </summary>
    private Text textComponent;
   
    // Use this for initialization
	void Start () {
        // Grab components
        textComponent = GetComponent<Text>();
        textComponent.text = "";
    }
	
	// Update is called once per frame
	void Update () {
	
	}

    /// <summary>
    /// Starts the countdown
    /// </summary>
    public void StartCountdown()
    {
        StartCoroutine(DoCountdown());
    }

    /// <summary>
    /// Displays a countdown, then enables can move
    /// </summary>
    /// <returns></returns>
    public IEnumerator DoCountdown()
    {
        yield return new WaitForSeconds(0.1f);
        int count = countdownText.Length;
        foreach(string s in countdownText)
        {
            if (textComponent) {
                PlayCountDownSound(--count);
                textComponent.text = s;
            }
            else { break; }
            yield return new WaitForSeconds(1);
        }
        textComponent.text = "";
        // Enalbe all players
        PlayerMotor[] players = FindObjectsOfType<PlayerMotor>();
        foreach(PlayerMotor p in players)
        {
            p.canMove = true;
        }
    }

    /// <summary>
    /// Plays the countdown sound. 
    /// Plays a different higher pitch on 0.
    /// </summary>
    /// <param name="count">The count at which the countdown timer is.</param>
    public void PlayCountDownSound(int count)
    {
        if (goSound == null || numbersSound == null)
        {
            return;
        }
        AudioSource.PlayClipAtPoint((count == 0) ? goSound : numbersSound, Vector3.zero);
    }

    /// <summary>
    /// Shows the round winning player on the screen.
    /// </summary>
    /// <param name="winningPlayer">The winning player</param>
    public void ShowRoundWinner(Player winningPlayer, Action callback)
    {
        string textColor = HexColorConverter.ColorToHex(winningPlayer.Manager.color);
        string text = "Player <color=#" + textColor + ">" + winningPlayer.ID + "</color> has won the round!";
        StartCoroutine(ShowText(text, 2.0f, callback));
    }

    /// <summary>
    /// Shows the game winning player on the screen.
    /// </summary>
    /// <param name="winningPlayer">The winning player</param>
    public void ShowGameWinner(Player winningPlayer, Action callback)
    {
        string textColor = HexColorConverter.ColorToHex(winningPlayer.Manager.color);
        string text = "Player <color=#" + textColor + ">" + winningPlayer.ID + "</color> has won the game!";
        StartCoroutine(ShowText(text, 4.0f, callback));
    }

    /// <summary>
    /// Show the winning player on the screen.
    /// </summary>
    /// <param name="winningPlayer">The winning player</param>
    /// <returns></returns>
    public IEnumerator ShowText(string text, float seconds, Action callback)
    {
        textComponent.text = text;
        yield return new WaitForSeconds(seconds);
        textComponent.text = "";
        callback.Invoke();
    }
}
