﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

using XboxCtrlrInput;
using Settings;

public class PlayerInputMethodController : MonoBehaviour {

    /// <summary>
    /// The screen mangers for the 6 players
    /// </summary>
    public PlayerSelectScreenManager[] selectionScreens;

    /// <summary>
    /// The text that can display info to the users
    /// </summary>
    public Text infoText;

    /// <summary>
    /// The current occupied types.
    /// </summary>
    [HideInInspector]
    public InputType[] inputTypes;

    // Boolean flags for each of the input
    bool hasMouseInput = false;
    bool hasKeyboardInput = false;
    bool[] controllerInput = {
        false, false, false, false
    };

    // Amount of players
    int playerCount = 0;

    // Use this for initialization
    void Start ()
    {
        infoText.text = "You need at lest 2 Players.Press A, S, RMB to join.";
    }
	
	// Update is called once per frame
	void Update ()
    {
        // Add player
        if (Input.GetMouseButtonDown(0) && !hasMouseInput)
        {
            hasMouseInput = true;
            AddPlayerInput(InputType.Mouse);
        }
        else if (Input.GetKeyDown(KeyCode.S) && !hasKeyboardInput)
        {
            hasKeyboardInput = true;
            AddPlayerInput(InputType.Keyboard);
        }
        for (int i = 0; i < 4; i++)
        {
            if (XCI.IsPluggedIn(i + 1) && controllerInput[i] == false)
            {
                if (XCI.GetButtonDown(XboxButton.A, i + 1))
                {
                
                    controllerInput[i] = true;
                    AddPlayerInput(InputType.Controller);
                }
            }
        }

        // Remove player
        if (Input.GetMouseButtonDown(1) && hasMouseInput)
        {
            hasMouseInput = false;
            RemovePlayerInput(InputType.Mouse);
        }
        else if (Input.GetKeyDown(KeyCode.E) && hasKeyboardInput)
        {
            hasKeyboardInput = false;
            RemovePlayerInput(InputType.Keyboard);
        }
        for (int i = 0; i < 4; i++)
        {
            if (XCI.IsPluggedIn(i + 1) && controllerInput[i] == true)
            {
                if (XCI.GetButtonDown(XboxButton.B, i + 1))
                {
                    controllerInput[i] = false;
                    RemovePlayerInput(InputType.Controller);
                }
            }
        }

        // Start game
        if (playerCount >= 2 && 
            (Input.GetKeyDown(KeyCode.Return) || XCI.GetButtonDown(XboxButton.Start)))
        {
            if (hasKeyboardInput)
            {
                if (PlayerSettings.Instance == null)
                {
                    GameObject go = new GameObject("Player Settings", typeof(PlayerSettings));
                }
                PlayerSettings.Instance.inputs.Add(InputType.Keyboard);
            }
            if (hasMouseInput)
            {
                if (PlayerSettings.Instance == null)
                {
                    GameObject go = new GameObject("Player Settings", typeof(PlayerSettings));
                }
                PlayerSettings.Instance.inputs.Add(InputType.Mouse);
            }
            for (int i = 0; i < 4; i++)
            {
                if (controllerInput[i])
                {
                    if (PlayerSettings.Instance == null)
                    {
                        GameObject go = new GameObject("Player Settings", typeof(PlayerSettings));
                    }
                    PlayerSettings.Instance.inputs.Add(InputType.Controller);
                    PlayerSettings.Instance.addedControllerIDs.Add(i);
                }
            }
            SceneManager.LoadScene("main");
        }
    }
    
    /// <summary>
    /// Adds an input by type
    /// </summary>
    /// <param name="type"></param>
    void AddPlayerInput(InputType type)
    {
        for (int i = 0; i < 6; i++)
        {
            if (inputTypes[i] == InputType.None)
            {
                playerCount++;
                if (playerCount > 1)
                {
                    infoText.text = "Press START or ENTER to start the game!";
                }
                selectionScreens[i].SetInputActive(type);
                inputTypes[i] = type;
                return;
            }
        }
    }

    /// <summary>
    /// Removes an input
    /// </summary>
    /// <param name="type"></param>
    void RemovePlayerInput(InputType type)
    {
        for (int i = 0; i < 6; i++)
        {
            if (inputTypes[i] == type)
            {
                if (playerCount < 2)
                {
                    infoText.text = "You need at lest 2 Players.Press A, S, RMB to join.";
                }
                playerCount--;
                inputTypes[i] = InputType.None;
                selectionScreens[i].SetInputDisabled(type);
            }
        }
    }
}
