﻿using UnityEngine;
using System.Collections;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class RGBEffect : UnityStandardAssets.ImageEffects.PostEffectsBase
{
    public Vector2 redShift = new Vector2(0.005f, 0f);
    public Vector2 greenShift = new Vector2(0f, 0.005f);
    public Vector2 blueShift = new Vector2(-0.005f, 0f);

    public Shader rgbShader = null;
    private Material rgbMaterial = null;

    public override bool CheckResources()
    {
        CheckSupport(false);

        rgbMaterial = CheckShaderAndCreateMaterial(rgbShader, rgbMaterial);

        if (!isSupported)
            ReportAutoDisable();
        return isSupported;
    }

    void OnDisable()
    {
        if (rgbMaterial)
            DestroyImmediate(rgbMaterial);
    }

    void OnRenderImage(RenderTexture source, RenderTexture destination)
    {
        if (CheckResources() == false)
        {
            Graphics.Blit(source, destination); //skip pass
            return;
        }
        rgbMaterial.SetVector("_RedShift", redShift);
        rgbMaterial.SetVector("_GreenShift", greenShift);
        rgbMaterial.SetVector("_BlueShift", blueShift);

        Graphics.Blit(source, destination, rgbMaterial);
    }
}