﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections.Generic;

public class ScoreContainerController : MonoBehaviour {

    /// <summary>
    /// A score viewer object, contains an icon and text.
    /// </summary>
    class ScoreViewer
    {
        public Image ScoreIcon;
        public Text ScoreText;

        public ScoreViewer(Image image, Text text, Color color)
        {
            ScoreIcon = image;
            ScoreText = text;
            ScoreIcon.color = color;
            ScoreText.color = color;
        }

        public void SetScore(int score)
        {
            if (ScoreText)
            {
                string s = "";
                for (int i = 0; i < score; i++)
                {
                    s += "|";
                }
                ScoreText.text = s;
            }
        }
    }

    public GameObject PlayerScoreViewerPrefab;

    Dictionary<Player, ScoreViewer> scoreViewers = 
        new Dictionary<Player, ScoreViewer>();

	/// <summary>
    /// Sets the score of a particular player
    /// </summary>
    /// <param name="player">The player that gets the score</param>
    /// <param name="score">The score that that player gets</param>
    public void SetScore(Player player)
    {
        if (player != null && scoreViewers.ContainsKey(player))
        {
            ScoreViewer viewer;
            scoreViewers.TryGetValue(player, out viewer);
            if (viewer != null)
            {
                viewer.SetScore(player.Score);
            }
        }
    }

    /// <summary>
    /// Initializes the score viewers for each player
    /// </summary>
    /// <param name="players">The list of players</param>
    public void InitializeScoreController(Player[] players)
    {
        foreach (Player player in players)
        {
            // Create new score viewer gameobject
            GameObject go = Instantiate(PlayerScoreViewerPrefab);
            go.transform.position = Vector3.zero;
            go.transform.localScale = Vector3.one;
            go.transform.SetParent(transform, false);

            // Create score viewer for every player
            ScoreViewer scoreViewer = new ScoreViewer(go.GetComponentInChildren<Image>(),
                                                        go.GetComponentInChildren<Text>(),
                                                        player.Manager.color);
            scoreViewer.SetScore(0);

            scoreViewers.Add(player, scoreViewer);
        }
    }
}
