﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;

using XboxCtrlrInput;


public class XboxMenuController : MonoBehaviour {

    /// <summary>
    /// The selection chosen at startup
    /// </summary>
    public Button firstSelection;

    /// <summary>
    /// The current selected selectable
    /// </summary>
    public Button currentSelection;

    /// <summary>
    /// How many times per second do we pol the controller axis?
    /// </summary>
    public int controllerPolFrequency = 5;

    /// <summary>
    /// Can we check the axis of the controller?
    /// </summary>
    private bool canCheckAxis = true;

    /// <summary>
    /// The current eventsystem
    /// </summary>
    EventSystem eventSystem;

    /// <summary>
    /// The player Id of the current frame
    /// </summary>
    int PlayerID = 0;

    // Use this for initialization
    void Start()
    {
        // Set first selected object if not done so
        if (firstSelection == null)
        {
            firstSelection = FindObjectOfType<Button>();
        }
        currentSelection = firstSelection;
        currentSelection.Select();

        // Pol the axis only with a certain frequency
        InvokeRepeating("SetCanCheckAxis", 0, 1.0f / (float)controllerPolFrequency);

        // Cache the event system
        eventSystem = FindObjectOfType<EventSystem>();
    }

    // Update is called once per frame
    void Update()
    {
        if (XCI.GetNumPluggedCtrlrs() == 0)
        {
            return;
        }

        PlayerID++;
        PlayerID = PlayerID % XCI.GetNumPluggedCtrlrs() + 1;
        
        // Check controller axises
        if (XCI.GetAxis(XboxAxis.LeftStickY, PlayerID) > 0.1f && canCheckAxis)
        {
            Button s = currentSelection.FindSelectableOnUp() as Button;
            if (s != null)
            {
                canCheckAxis = false;
                s.Select();
                currentSelection = s;
            }
        }
        else if (XCI.GetAxis(XboxAxis.LeftStickY, PlayerID) < -0.1f && canCheckAxis)
        {
            Button s = currentSelection.FindSelectableOnDown() as Button;
            if (s != null)
            {
                canCheckAxis = false;
                s.Select();
                currentSelection = s;
            }
        }
        else if (XCI.GetAxis(XboxAxis.LeftStickY, PlayerID) > -0.1f && XCI.GetAxis(XboxAxis.LeftStickY, PlayerID) < 0.1f)
        {
            canCheckAxis = true;
        }

        // Check for a button or b button
        if (XCI.GetButtonDown(XboxButton.A, PlayerID) && currentSelection.IsInteractable())
        {
            // DEBUG : currentSelection.onClick.AddListener(() => { print("I have been clicked! Yeahyj!"); });
            currentSelection.OnSubmit(new BaseEventData(eventSystem));  // Omfg why does this work?!
        }

        if (XCI.GetButtonDown(XboxButton.Back, PlayerID))
        {
            
        }

        if (XCI.GetButtonDown(XboxButton.Start, PlayerID))
        {
            
        }
    }

    /// <summary>
    /// Pol the controller for axis updates
    /// </summary>
    private void SetCanCheckAxis()
    {
        canCheckAxis = true;
    }

    /// <summary>
    /// When this menu is destroyed
    /// </summary>
    void OnDestroy()
    {
         
    }
}
